#ifndef CIPHER_H 
#define CIPHER_H

using namespace std;

#include <vector>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

//! Class for 26-letter ciphertexts, with language identification
class nzCipherText {
 private:
  //! the text proper
  string text;
  //! title of the ciphertext (eg NeMa ciphertext, english plaintext, ...)
  string name;
  //! letter frequencies of the text
  int freq[26];
  //! remove all non-alphabetical characters, convert all to lowercase
  string stripText(string strIn);
  //! identify language, based on minimal chi2
  string identify();
  //! return index of smallest vector entry
  int getMinIndex(vector<double> vect);

 public:
  //! default constructor
  nzCipherText();
  //! constructor with ciphertext title
  nzCipherText(string tName);
  //! constructor with text and title
  nzCipherText(string tText, string tName);
  //! clear text and frequencies
  void reset();
  //! set ciphertext title
  void setName(string tName) {name=tName;};
  //! add a character 
  void add(char addChar);
  //! add a character passing its alphabetical index [0-25]
  void add(int charCode);
  //! add a string
  void add(string strIn);
  //! add contents of file fName
  void addFile(const char* fName);
  //! print text in blocks of 5
  void blockPrint();
  //! get chi2 with respect to frequencies of the English language 
  double getEnglishChi2();
  //! get chi2 with respect to frequencies of the French language 
  double getFrenchChi2();
  //! get chi2 with respect to the frequency of random letters (1/26)
  double getUniformChi2();
  //! get chi2 with respect to the frequencies of a reflector-based cipher machine
  double getReflectorChi2();
  //! return text as string
  string str() {return text;};
  //! write frequency table to file
  void writeFreqTable(const char* outFile);
  
};

#endif

