#include "main.h"

int main(int argc, const char* argv[]) {
  if (argc<3) {
    cout << "usage: ./NeMa inputfile password\n" 
	 << "       ./NeMa -bf cipherfile\n"
	 << "       ./NeMa -kp plainfile cipherfile" << endl;
    return 1;
  }
  if (strcmp(argv[1], "-bf")==0) {
    // brute-force attack
    const char* inputfile = argv[2];
    bruteForce(inputfile);   
    return 0;
  }
  if (strcmp(argv[1], "-kp")==0) {
    // known-plaintext attack
    const char* plainfile  = argv[2];
    const char* cipherfile = argv[3];
    knownPlaintext(plainfile, cipherfile);   
    return 0;
  }

  // standard encryption
  const char* inputfile = argv[1];
  const char* password = argv[2];

  nzNema nema(password);

  nzCipherText plainText("NeMa input");
  plainText.addFile(inputfile);
  plainText.blockPrint();

  string cipher = nema.crypt(plainText);
  nzCipherText cipherText(cipher, "NeMa output");
  //  cipherText.writeFreqTable("nemafreq.data");
  //  plainText.writeFreqTable("enfreq.data");
  //  plainText.blockPrint();
}

void bruteForce(const char* inputfile) {
  nzNema nema;
  nzCipherText cipherText("brute force ciphertext");
  cipherText.addFile(inputfile);
  cipherText.blockPrint();
  long int nKeys = 0; 
  double chi2 = 10.;

  for (int pw0=18; pw0<44; pw0++) {
    for (int pw1=18; pw1<44; pw1++) {
      for (int pw2=18; pw2<44; pw2++) {
	for (int pw3=18; pw3<44; pw3++) {
	  for (int pw4=18; pw4<44; pw4++) {
	    for (int pw5=18; pw5<44; pw5++) {
	      for (int pw6=18; pw6<44; pw6++) {
		for (int pw7=18; pw7<44; pw7++) {
		  for (int pw8=18; pw8<44; pw8++) {
		    for (int pw9=18; pw9<44; pw9++) {
		      nKeys++;
		      int passwd[]={pw0, pw1, pw2, pw3, pw4, pw5, pw6, pw7, pw8, pw9};
		      for (int k=0; k<10; k++) passwd[k] %=26;
		      nema.initState(passwd);
		      if (nKeys % 1000 == 0) 
			cout << "trying key " << nKeys << " " << getPassword(passwd) << "\r"; 
		      chi2 = nema.fastCrypt(cipherText);
		      if (chi2 < 2.0) {
			double chi2un = nema.getUniformChi2();
			cout << "possible key found: " << getPassword(passwd) 
			     << " key " << nKeys << " chi2-en " << chi2 
			     << " chi2-un " << chi2un;
			if (chi2un > 2.5) cout << " good candidate";
			cout << endl;
		      }
		    }
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
  exit(0);
}

string getPassword(int pw[10]) {
  string passwd = "";
  for (int i=0; i<10; i++) passwd += char(((pw[i]+8) %26)+(int)'a');
  return passwd;
}

void knownPlaintext(const char* plainfile, const char* cipherfile) {
  nzNema nema;

  nzCipherText cipherText("kpta ciphertext");
  nzCipherText plainText("kpta plaintext");
  cipherText.addFile(cipherfile);
  plainText.addFile(plainfile);
  plainText.blockPrint();
  cipherText.blockPrint();
  long int nKeys = 0; 

  for (int pw0=18; pw0<44; pw0++) {
    for (int pw1=18; pw1<44; pw1++) {
      for (int pw2=18; pw2<44; pw2++) {
	for (int pw3=18; pw3<44; pw3++) {
	  for (int pw4=18; pw4<44; pw4++) {
	    for (int pw5=18; pw5<44; pw5++) {
	      for (int pw6=18; pw6<44; pw6++) {
		for (int pw7=18; pw7<44; pw7++) {
		  for (int pw8=18; pw8<44; pw8++) {
		    for (int pw9=18; pw9<44; pw9++) {
		      nKeys++;
		      int passwd[]={pw0, pw1, pw2, pw3, pw4, pw5, pw6, pw7, pw8, pw9};
		      for (int k=0; k<10; k++) passwd[k] %=26;
		      nema.initState(passwd);
		      if (nKeys % 1000 == 0) 
			cout << "trying key " << nKeys << " " << getPassword(passwd) << "\r"; 
		      if (nema.kpta(plainText.str(), cipherText.str())) {
			cout << "key found: " << getPassword(passwd) << " key " << nKeys << endl;
			return;
		      }
		    }
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
}

