#include "nemaWheels.h"
#include "nema.h"

nzNema::nzNema() {
  cout << "NeMa simulation by nicZ" << endl;
  setInnerKey();
  initState("aaaaaaaaaa");
  cipher.setName("NeMa output");
}
nzNema::nzNema(string passwd) {
  cout << "NeMa simulation by nicZ" << endl;
  setInnerKey();
  initState(passwd);
  cipher.setName("NeMa output");  
}

string nzNema::crypt(string strIn) {
  printState(); cout << endl;
  for (int k=0; k<strIn.length(); k++) {
    stepState();
    printState();
    char charOut = flow(strIn[k]);
    cout << " in " << strIn[k] << " out " << charOut << endl;
    cipher.add(charOut);
  }
  cipher.blockPrint();
  return cipher.str();
}

double nzNema::fastCrypt(string strIn) {
  for (int k=0; k<strIn.length(); k++) {
    stepState();
    cipher.add(flow(strIn[k]));
  }
  return cipher.getEnglishChi2();
}

bool nzNema::kpta(const string strPlain, const string strCipher) {
  for (int k=0; k<strCipher.length(); k++) {
    stepState();
    if (flow(strCipher[k]) != strPlain[k]) return false;
  }
  return true;
}

char nzNema::flow(char charIn) {
  int channel = keybd[(int)charIn-(int)'a'];
  channel = (channel+24) %26;
  int offsets[5];
  for (int k=0; k<5; k++) offsets[k]=getWheelOffset(k);

  for (int i=0; i<4; i++) channel = rotors[i][(channel+offsets[i]+26) % 26];
  channel = ukw[(channel+offsets[4]+26) % 26];
  for (int i=3; i>=0; i--) channel = invrot[i][(channel-offsets[i+1]+26) % 26];
  channel = (channel-offsets[0]+26) % 26;
  
  return lamps[(26-channel+24) %26];
}

void nzNema::setInnerKey() {
  notchRings[0] = nr1;
  notchRings[1] = nr22;
  notchRings[2] = nr15;
  notchRings[3] = nr14;
  notchRings[4] = nr13;
  notchRings[5] = nr12;

  rotors[0] = rwD;
  rotors[1] = rwC;
  rotors[2] = rwB;
  rotors[3] = rwA;
  invrot[0] = irwD;
  invrot[1] = irwC;
  invrot[2] = irwB;
  invrot[3] = irwA;
}

int nzNema::getWheelOffset(int wheel) {
  if (wheel == 0) return state[1];
  return (state[2*wheel+1] - state[2*wheel-1] +26) % 26;
}

void nzNema::initState(string passwd) {
  steps = 0;
  cipher.reset();
  if (passwd.length()<10) {
    cout << "Error: password " << passwd << " is too short!" << endl;
    exit(0);
  }
  for (int i=0; i<10; i++) {
    char lcp = tolower(passwd[9-i]);
    if (lcp<'a' || lcp>'z') {
      cout << "Error: invalid password " << passwd << endl;
      cout << "       password should contain only letters [a-z] or [A-Z]" << endl;
      exit(0);
    }
    state[i] = ((int)lcp -(int)'a' +18) %26;
  }
}

void nzNema::initState(int init[10]) {
  steps = 0;
  cipher.reset();
  //  cout << "initialize state" << endl;
  for (int i=0; i<10; i++) state[i] = init[9-i];
  //printState(); cout << endl;
}

void nzNema::stepState() {
  int state0 = notchRings[1][(state[0]+24) % 26];
  int state1 = notchRings[0][(state[0]+25) % 26];
  int state2 = notchRings[2][(state[2]+24) % 26];
  int state4 = notchRings[3][(state[4]+24) % 26];
  int state6 = notchRings[4][(state[6]+24) % 26];
  int state8 = notchRings[5][(state[8]+24) % 26];

  steps++;
  state[0] = (state[0]+25) % 26;
  state[4] = (state[4]+25) % 26;
  state[8] = (state[8]+25) % 26;

  if (state0==1) state[1] = (state[1]+25) % 26;
  if (state4==1) state[5] = (state[5]+25) % 26;
  if (state8==1) state[9] = (state[9]+25) % 26;
  
  if (state1==1) {
    state[2] = (state[2]+25) % 26;
    state[6] = (state[6]+25) % 26;
    if (state2==1) state[3] = (state[3]+25) % 26;
    if (state6==1) state[7] = (state[7]+25) % 26;
  }
}

void nzNema::printState() {
  cout << "step " << steps << " state ";
  for (int i=9; i>-1; i--) cout << char(((state[i]+8) %26) + 'A');
}
